# ----------------------------------------------------------------------------
# Copyright 2021 Pelion
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ----------------------------------------------------------------------------
# Auto-generated by asn1ate v.0.6.0 from package.asn
# (last modified on 2021-06-24 10:25:24.529484)

from pyasn1.type import univ, char, namedtype, constraint


class Bytes(univ.OctetString):
    pass


class Int(univ.Integer):
    pass


class String(char.UTF8String):
    pass


class ImgDescriptor(univ.Sequence):
    pass


ImgDescriptor.componentType = namedtype.NamedTypes(
    namedtype.NamedType('id', String()),
    namedtype.NamedType('vendor-data', Bytes()),
    namedtype.NamedType('vendor-data-size', Int()),
    namedtype.NamedType('image-size', Int())
)


class Descriptor(univ.Sequence):
    pass


Descriptor.componentType = namedtype.NamedTypes(
    namedtype.NamedType('num-of-images', \
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, 255))),
    namedtype.NamedType('descriptors-array', univ.SequenceOf(componentType=ImgDescriptor()))
)
